==> docker-exec-websocket-server-1.0.11.tgz: package/lib/server.js <==
'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _get = require('babel-runtime/helpers/get')['default'];

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

var _ = require('lodash');
var assert = require('assert');
var debug = require('debug')('docker-exec-websocket-server:lib:server');
var debugdata = require('debug')('docker-exec-websocket-server:lib:sent');
var Docker = require('dockerode-promise');
var events = require('events');
var fs = require('fs');
var msgcode = require('./messagecodes.js');
var slugid = require('slugid');
var through = require('through');
var url = require('url');
var ws = require('ws');

var ExecSession = (function () {
  function ExecSession(options) {
    _classCallCheck(this, ExecSession);

    this.options = options;
    this.execOptions = {
      AttachStdout: true,
      AttachStderr: true,
      AttachStdin: true,
      Tty: options.tty,
      Detach: false,
      Cmd: options.command
    };
    this.attachOptions = {
      stdin: true,
      stdout: true,
      stderr: true,
      stream: true
    };
    this.container = options.container;
    this.socket = options.socket;
    this.server = options.server;
  }

  _createClass(ExecSession, [{
    key: 'execute',
    value: function execute() {
      var MAX_OUTSTANDING_BYTES;
      return _regeneratorRuntime.async(function execute$(context$2$0) {
        var _this = this;

        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            context$2$0.next = 2;
            return _regeneratorRuntime.awrap(this.container.exec(this.execOptions));

          case 2:
            this.exec = context$2$0.sent;
            context$2$0.next = 5;
            return _regeneratorRuntime.awrap(this.exec.start(this.attachOptions));

          case 5:
            this.execStream = context$2$0.sent;

            //handling output
            this.strout = through(function (data) {
              _this.sendMessage(msgcode.stdout, new Buffer(data));
            });

            this.strerr = through(function (data) {
              _this.sendMessage(msgcode.stderr, new Buffer(data));
            });

            this.exec.modem.demuxStream(this.execStream, this.strout, this.strerr);
            //This stream is created solely for the purposes of pausing, because
            //data will only buffer up in streams using this.queue()
            this.strbuf = through();

            MAX_OUTSTANDING_BYTES = 8 * 1024 * 1024;

            this.outstandingBytes = 0;

            this.strbuf.pipe(through(function (data) {
              _this.outstandingBytes += data.length;
              debugdata(data);
              _this.socket.send(data, { binary: true }, function () {
                _this.outstandingBytes -= data.length;
              });
              if (_this.outstandingBytes > MAX_OUTSTANDING_BYTES) {
                _this.strbuf.pause();
              } else {
                _this.strbuf.resume();
              }
            }));

            //handling input
            this.socket.on('message', function (message) {
              _this.messageHandler(message);
            });

            this.socket.on('disconnect', function () {
              //should be how to ctrl+c ctrl+d, might be better way to kill
              _this.execStream.end('\u0003\u0004\r\nexit\r\n');
              debug('client disconnect');
              //for now, it kills this session
              _this.close();
            });

            this.socket.on('close', function () {
              //should be how to ctrl+c ctrl+d, might be better way to kill
              _this.execStream.end('\u0003\u0004\r\nexit\r\n');
              debug('client close');
              //for now, it kills this session
              _this.close();
            });

            //start recieving client output again
            this.execStream.on('drain', function () {
              _this.sendCode(msgcode.resume);
              debug('resumed');
            });

            //When the process dies, the stream ends
            this.execStream.on('end', function () {
              _this.execStreamEnd();
            });

            //send ready message to let client know it is ready
            this.sendCode(msgcode.resume);
            debug('server finished executing session');

          case 20:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }
  }, {
    key: 'sendCode',
    value: function sendCode(code) {
      this.strbuf.write(new Buffer([code]), { binary: true });
    }
  }, {
    key: 'sendMessage',
    value: function sendMessage(code, buffer) {
      this.strbuf.write(Buffer.concat([new Buffer([code]), buffer]), { binary: true });
    }
  }, {
    key: 'messageHandler',
    value: function messageHandler(message) {
      switch (message[0]) {
        case msgcode.pause:
          this.strbuf.pause();
          debug('paused');
          break;

        case msgcode.resume:
          this.strbuf.resume();
          debug('resumed');
          break;

        case msgcode.stdin:
          if (!this.execStream.write(message.slice(1), { binary: true })) {
            this.sendCode(msgcode.pause);
            debug('paused');
          }
          break;

        case msgcode.end:
          this.execStream.end();
          break;

        default:
          debug('unknown msg code %s', message[0]);
      }
    }
  }, {
    key: 'execStreamEnd',
    value: function execStreamEnd() {
      var _this2 = this;

      this.exec.inspect().then(function (data) {
        debug('%s is exit code', data.ExitCode);
        _this2.sendMessage(msgcode.stopped, new Buffer([data.ExitCode]));
        _this2.close();
      }, function () {
        _this2.forceClose();
      });
    }
  }, {
    key: 'forceClose',
    value: function forceClose() {
      this.sendCode(msgcode.shutdown);
      this.close();
    }
  }, {
    key: 'close',
    value: function close() {
      var _this3 = this;

      this.server.sessions.splice(this.server.sessions.indexOf(this), 1);
      debug('%s sessions remain', this.server.sessions.length);
      this.server.emit('session removed', this.server.sessions.length);

      if (!this.strbuf.paused) {
        this.socket.close();
        this.strout.end();
        this.strerr.end();
        this.strbuf.end();
      } else {
        this.strbuf.on('drain', function () {
          _this3.socket.close();
          _this3.strout.end();
          _this3.strerr.end();
          _this3.strbuf.end();
        });
      }
    }
  }]);

  return ExecSession;
})();

var DockerExecWebsocketServer = (function (_events$EventEmitter) {
  /* Creates Docker Exec instance on given container, running the first message given
   * as a command.
   * Options:
   * port, required
   * OR
   * server, instance of http.Server or https.Server already listening on a port
   * containerId, name or id of docker container, required
   * path, path where the websocket is hosted
   * dockerSocket, path to docker's remote API
   * maxSessions, the maximum number of sessions allowed for one server
   * wrapperCommand, an optional wrapper script which wraps the command query
   */

  function DockerExecWebsocketServer(options) {
    var _this4 = this;

    _classCallCheck(this, DockerExecWebsocketServer);

    _get(Object.getPrototypeOf(DockerExecWebsocketServer.prototype), 'constructor', this).call(this);
    this.options = options = _.defaults({}, options, { path: '/' + slugid.v4(),
      dockerSocket: '/var/run/docker.sock',
      maxSessions: 10
    });

    //setting up docker
    var stats = fs.statSync(options.dockerSocket);
    if (!stats.isSocket()) {
      throw new Error('Are you sure the docker is running?');
    }
    var docker = new Docker({ socketPath: options.dockerSocket });

    //getting container
    assert(options.containerId, 'required container option missing');
    var container = docker.getContainer(options.containerId);
    assert(container, 'could not get container from Docker');

    //making websocket server
    var wsopts;
    if (options.server) {
      wsopts = {
        server: options.server,
        path: options.path
      };
    } else if (options.port && options.path) {
      wsopts = {
        port: options.port,
        path: options.path
      };
    }
    assert(wsopts, 'required port or server option missing');
    this.server = new ws.Server(wsopts);
    if (options.port && options.path) {
      debug('%s%s created', wsopts.port, wsopts.path);
    } else {
      debug('websocket server created');
    }

    this.sessions = [];

    this.server.on('connection', function (socket) {
      debug('connection recieved');
      if (_this4.sessions.length < _this4.options.maxSessions) {
        var args = url.parse(socket.upgradeReq.url, true).query;
        if (typeof args.command === 'string') {
          args.command = [args.command];
        }
        var session = new ExecSession({
          container: container,
          socket: socket,
          command: (options.wrapperCommand ? options.wrapperCommand : []).concat(args.command),
          tty: /^true$/i.test(args.tty),
          server: _this4
        });
        _this4.sessions.push(session);
        session.execute();
        debug('%s sessions created', _this4.sessions.length);
        _this4.emit('session added', _this4.sessions.length);
      } else {
        socket.send(Buffer.concat([new Buffer([msgcode.error]), new Buffer('Too many sessions active!')]));
      }
    });
  }

  _inherits(DockerExecWebsocketServer, _events$EventEmitter);

  _createClass(DockerExecWebsocketServer, [{
    key: 'close',
    value: function close() {
      this.server.close();
      this.sessions.forEach(function (session) {
        session.forceClose();
      });
    }
  }]);

  return DockerExecWebsocketServer;
})(events.EventEmitter);

exports['default'] = DockerExecWebsocketServer;
module.exports = exports['default'];

//TODO: add error handling support here

==> docker-exec-websocket-server-1.0.12.tgz: package/lib/server.js <==
'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _get = require('babel-runtime/helpers/get')['default'];

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

var _ = require('lodash');
var assert = require('assert');
var debug = require('debug')('docker-exec-websocket-server:lib:server');
var debugdata = require('debug')('docker-exec-websocket-server:lib:sent');
var Docker = require('dockerode-promise');
var events = require('events');
var fs = require('fs');
var msgcode = require('./messagecodes.js');
var slugid = require('slugid');
var through = require('through');
var url = require('url');
var ws = require('ws');

var ExecSession = (function () {
  function ExecSession(options) {
    _classCallCheck(this, ExecSession);

    this.options = options;
    this.execOptions = {
      AttachStdout: true,
      AttachStderr: true,
      AttachStdin: true,
      Tty: options.tty,
      Detach: false,
      Cmd: options.command
    };
    this.attachOptions = {
      stdin: true,
      stdout: true,
      stderr: true,
      stream: true
    };
    this.container = options.container;
    this.socket = options.socket;
    this.server = options.server;
  }

  _createClass(ExecSession, [{
    key: 'execute',
    value: function execute() {
      var MAX_OUTSTANDING_BYTES;
      return _regeneratorRuntime.async(function execute$(context$2$0) {
        var _this = this;

        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            context$2$0.next = 2;
            return _regeneratorRuntime.awrap(this.container.exec(this.execOptions));

          case 2:
            this.exec = context$2$0.sent;
            context$2$0.next = 5;
            return _regeneratorRuntime.awrap(this.exec.start(this.attachOptions));

          case 5:
            this.execStream = context$2$0.sent;

            //handling output
            this.strout = through(function (data) {
              _this.sendMessage(msgcode.stdout, new Buffer(data));
            });

            this.strerr = through(function (data) {
              _this.sendMessage(msgcode.stderr, new Buffer(data));
            });

            this.exec.modem.demuxStream(this.execStream, this.strout, this.strerr);
            //This stream is created solely for the purposes of pausing, because
            //data will only buffer up in streams using this.queue()
            this.strbuf = through();

            MAX_OUTSTANDING_BYTES = 8 * 1024 * 1024;

            this.outstandingBytes = 0;

            this.strbuf.pipe(through(function (data) {
              _this.outstandingBytes += data.length;
              debugdata(data);
              _this.socket.send(data, { binary: true }, function () {
                _this.outstandingBytes -= data.length;
              });
              if (_this.outstandingBytes > MAX_OUTSTANDING_BYTES) {
                _this.strbuf.pause();
              } else {
                _this.strbuf.resume();
              }
            }));

            //handling input
            this.socket.on('message', function (message) {
              _this.messageHandler(message);
            });

            this.socket.on('close', function () {
              //should be how to ctrl+c ctrl+d, might be better way to kill
              _this.execStream.end('\u0003\u0004\r\nexit\r\n');
              debug('client close');
              //for now, it kills this session
              _this.close();
            });

            //start recieving client output again
            this.execStream.on('drain', function () {
              _this.sendCode(msgcode.resume);
              debug('resumed');
            });

            //When the process dies, the stream ends
            this.execStream.on('end', function () {
              _this.execStreamEnd();
            });

            //send ready message to let client know it is ready
            this.sendCode(msgcode.resume);
            debug('server finished executing session');

          case 19:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }
  }, {
    key: 'sendCode',
    value: function sendCode(code) {
      this.strbuf.write(new Buffer([code]), { binary: true });
    }
  }, {
    key: 'sendMessage',
    value: function sendMessage(code, buffer) {
      this.strbuf.write(Buffer.concat([new Buffer([code]), buffer]), { binary: true });
    }
  }, {
    key: 'messageHandler',
    value: function messageHandler(message) {
      switch (message[0]) {
        case msgcode.pause:
          this.strbuf.pause();
          debug('paused');
          break;

        case msgcode.resume:
          this.strbuf.resume();
          debug('resumed');
          break;

        case msgcode.stdin:
          if (!this.execStream.write(message.slice(1), { binary: true })) {
            this.sendCode(msgcode.pause);
            debug('paused');
          }
          break;

        case msgcode.end:
          this.execStream.end();
          break;

        default:
          debug('unknown msg code %s', message[0]);
      }
    }
  }, {
    key: 'execStreamEnd',
    value: function execStreamEnd() {
      var _this2 = this;

      this.exec.inspect().then(function (data) {
        debug('%s is exit code', data.ExitCode);
        _this2.sendMessage(msgcode.stopped, new Buffer([data.ExitCode]));
        _this2.close();
      }, function () {
        _this2.forceClose();
      });
    }
  }, {
    key: 'forceClose',
    value: function forceClose() {
      this.sendCode(msgcode.shutdown);
      this.close();
    }
  }, {
    key: 'close',
    value: function close() {
      var _this3 = this;

      var index = this.server.sessions.indexOf(this);
      if (index > 0) {
        this.server.sessions.splice(index, 1);
        debug('%s sessions remain', this.server.sessions.length);
        this.server.emit('session removed', this.server.sessions.length);

        if (!this.strbuf.paused) {
          this.socket.close();
          this.strout.end();
          this.strerr.end();
          this.strbuf.end();
        } else {
          this.strbuf.on('drain', function () {
            _this3.socket.close();
            _this3.strout.end();
            _this3.strerr.end();
            _this3.strbuf.end();
          });
        }
      }
    }
  }]);

  return ExecSession;
})();

var DockerExecWebsocketServer = (function (_events$EventEmitter) {
  /* Creates Docker Exec instance on given container, running the first message given
   * as a command.
   * Options:
   * port, required
   * OR
   * server, instance of http.Server or https.Server already listening on a port
   * containerId, name or id of docker container, required
   * path, path where the websocket is hosted
   * dockerSocket, path to docker's remote API
   * maxSessions, the maximum number of sessions allowed for one server
   * wrapperCommand, an optional wrapper script which wraps the command query
   */

  function DockerExecWebsocketServer(options) {
    var _this4 = this;

    _classCallCheck(this, DockerExecWebsocketServer);

    _get(Object.getPrototypeOf(DockerExecWebsocketServer.prototype), 'constructor', this).call(this);
    this.options = options = _.defaults({}, options, { path: '/' + slugid.v4(),
      dockerSocket: '/var/run/docker.sock',
      maxSessions: 10
    });

    //setting up docker
    var stats = fs.statSync(options.dockerSocket);
    if (!stats.isSocket()) {
      throw new Error('Are you sure the docker is running?');
    }
    var docker = new Docker({ socketPath: options.dockerSocket });

    //getting container
    assert(options.containerId, 'required container option missing');
    var container = docker.getContainer(options.containerId);
    assert(container, 'could not get container from Docker');

    //making websocket server
    var wsopts;
    if (options.server) {
      wsopts = {
        server: options.server,
        path: options.path
      };
    } else if (options.port && options.path) {
      wsopts = {
        port: options.port,
        path: options.path
      };
    }
    assert(wsopts, 'required port or server option missing');
    this.server = new ws.Server(wsopts);
    if (options.port && options.path) {
      debug('%s%s created', wsopts.port, wsopts.path);
    } else {
      debug('websocket server created');
    }

    this.sessions = [];

    this.server.on('connection', function (socket) {
      debug('connection recieved');
      if (_this4.sessions.length < _this4.options.maxSessions) {
        var args = url.parse(socket.upgradeReq.url, true).query;
        if (typeof args.command === 'string') {
          args.command = [args.command];
        }
        var session = new ExecSession({
          container: container,
          socket: socket,
          command: (options.wrapperCommand ? options.wrapperCommand : []).concat(args.command),
          tty: /^true$/i.test(args.tty),
          server: _this4
        });
        _this4.sessions.push(session);
        session.execute();
        debug('%s sessions created', _this4.sessions.length);
        _this4.emit('session added', _this4.sessions.length);
      } else {
        socket.send(Buffer.concat([new Buffer([msgcode.error]), new Buffer('Too many sessions active!')]));
      }
    });
  }

  _inherits(DockerExecWebsocketServer, _events$EventEmitter);

  _createClass(DockerExecWebsocketServer, [{
    key: 'close',
    value: function close() {
      this.server.close();
      this.sessions.forEach(function (session) {
        session.forceClose();
      });
    }
  }]);

  return DockerExecWebsocketServer;
})(events.EventEmitter);

exports['default'] = DockerExecWebsocketServer;
module.exports = exports['default'];

//TODO: add error handling support here
